from shapely.geometry.base import BaseGeometry

from .morphology import closing, opening

__all__ = ["zero_buffer", "deflimmer", "clean_geometry", "unflimmer", "sanitise"]


def zero_buffer(
    geom: BaseGeometry,
) -> BaseGeometry:
    return geom.buffer(0)


def deflimmer(geom: BaseGeometry, eps: float = 1e-7) -> BaseGeometry:
    """

    :param geom:
    :param eps:
    :return:
    """
    return opening(closing(geom, eps), eps)


clean_geometry = unflimmer = deflimmer


def sanitise(geom: BaseGeometry, *args: callable) -> BaseGeometry:
    """
      #A positive distance produces a dilation, a negative distance an erosion. A very small or zero distance may sometimes be used to “tidy” a polygon.

    :param geom:
    :param args:
    :return:
    """

    if not len(args):
        args = (zero_buffer, deflimmer)

    for f in args:
        geom = f(geom)

    return geom



#!/usr/bin/env python3

__author__ = "Christian Heider Lindbjerg"
__doc__ = r"""

           Created on 02-12-2020
           """

from typing import Any, Optional, Tuple

# noinspection PyUnresolvedReferences
from qgis.PyQt import QtCore

__all__ = ["MyTableModel"]


# noinspection PyPep8Naming
class MyTableModel(QtCore.QAbstractTableModel):
    """
    A model that can be used to display a table of data.

    :param data: The data to display.
    :param parent: The parent object.

    """

    def __init__(self, data: Tuple = (()), parent: Any = None):
        super().__init__(parent)
        self.data = data

    def headerData(
        self, section: int, orientation: QtCore.Qt.Orientation, role: int
    ) -> str:  # Do not rename
        """

        :param section:
        :param orientation:
        :param role:
        :return:
        """
        if role == QtCore.Qt.DisplayRole:
            if orientation == QtCore.Qt.Horizontal:
                return f"Column {str(section)}"
            else:
                return f"Row {str(section)}"

    def columnCount(self, parent: Any = None) -> int:  # Do not rename
        """

        :param parent:
        :return:
        """
        if self.rowCount():
            return len(self.data[0])
        return 0

    def rowCount(self, parent: Any = None) -> int:
        """

        :param parent:
        :return:
        """
        return len(self.data)

    def data(self, index: QtCore.QModelIndex, role: int) -> Optional[str]:
        """

        :param index:
        :param role:
        :return:
        """
        if role == QtCore.Qt.DisplayRole:
            row = index.row()
            col = index.column()
            return str(self.data[row][col])



#!/usr/bin/env python3

__author__ = "heider"
__doc__ = r"""

            This module must remain clean and clear of additional external dependencies outside of qgis, and base requirements of jord.

           Created on 5/5/22
           """

import logging
from pathlib import Path

with open(Path(__file__).parent / "README.md") as this_init_file:
    __doc__ += this_init_file.read()

try:
    from .importing import *
    from .configuration import *

    # from .helpers import * # import issues
    # from .numpy_utilities import *
    from .data_provider import *
    from .geometry_types import *
    from .conversion import *

    # from .categorisation import *
    # from .styling import *
    # from .plugin_version import *
except ImportError as ix:
    this_package_name = Path(__file__).parent.name
    logging.error(f"Make sure qgis module is available for {this_package_name}")
    raise ix



#!/usr/bin/env python3
from pathlib import Path

try:
    from .enums import *
except ImportError as ix:
    this_package_name = Path(__file__).parent.name
    print(f"Make sure qt module is available for {this_package_name}")
    raise ix



#!/usr/bin/env python3
from typing import Mapping, Any

import shapely
from networkx import MultiDiGraph

__all__ = ["assertive_add_edge", "add_shapely_node"]


def assertive_add_edge(
    graph: MultiDiGraph,
    u: int,
    v: int,
    uniqueid: int,
    attributes: Mapping[str, Any],
    *,
    allow_loops: bool = True,
    allow_duplicates: bool = False,
) -> None:
    if not allow_loops:
        assert u != v, f"{u} == {v}"

    assert isinstance(u, int)
    assert isinstance(v, int)

    assert graph.has_node(u)
    assert graph.has_node(v)

    if not allow_duplicates and graph.has_edge(u, v, uniqueid):
        assert not graph.has_edge(u, v, uniqueid)

    graph.add_edge(u, v, key=uniqueid, uniqueid=uniqueid, **attributes)


def add_shapely_node(
    graph: MultiDiGraph, u: int, point: shapely.Point, **kwargs
) -> None:
    assert isinstance(u, int)

    graph.add_node(
        u,
        lon=point.x,
        lat=point.y,
        x=point.x,
        y=point.y,
        **kwargs,
    )



#!/usr/bin/env python3

import shapely
from shapely.geometry.base import BaseGeometry

__all__ = ["closing", "opening", "erode", "erosion", "dilate", "dilation", "close"]

from warg import passes_kws_to

FALLBACK_CAPSTYLE = shapely.BufferCapStyle.round  # CAN BE OVERRIDEN


@passes_kws_to(shapely.geometry.base.BaseGeometry.buffer)
def erosion(
    geom: BaseGeometry,
    distance: float = 1e-7,
    cap_style: shapely.BufferCapStyle = shapely.BufferCapStyle.flat,
    join_style: shapely.BufferJoinStyle = shapely.BufferJoinStyle.mitre,
    **kwargs
) -> BaseGeometry:
    """

    :param distance:
    :param cap_style:
    :param join_style:
    :param geom: The geometry to be eroded
    :return: The eroded geometry
    """
    return geom.buffer(
        distance=-distance, cap_style=cap_style, join_style=join_style, **kwargs
    )


@passes_kws_to(shapely.geometry.base.BaseGeometry.buffer)
def dilation(
    geom: BaseGeometry,
    distance: float = 1e-7,
    cap_style: shapely.BufferCapStyle = shapely.BufferCapStyle.flat,
    join_style: shapely.BufferJoinStyle = shapely.BufferJoinStyle.mitre,
    **kwargs
) -> BaseGeometry:
    """

    :param cap_style:
    :param join_style:
    :param geom: The geometry to be dilated
    :param distance: Dilation amount
    :return: The dilated geometry
    """
    if (
        isinstance(geom, shapely.Point) and cap_style == shapely.BufferCapStyle.flat
    ):  # NONSENSE, propably not what is intended
        cap_style = FALLBACK_CAPSTYLE

    return geom.buffer(
        distance=distance, cap_style=cap_style, join_style=join_style, **kwargs
    )


@passes_kws_to(shapely.geometry.base.BaseGeometry.buffer)
def closing(geom: BaseGeometry, **kwargs) -> BaseGeometry:
    """

    :param geom: The geometry to be closed
    :return: The closed geometry
    """
    return erode(dilate(geom, **kwargs), **kwargs)


@passes_kws_to(shapely.geometry.base.BaseGeometry.buffer)
def opening(geom: BaseGeometry, **kwargs) -> BaseGeometry:
    """

    :param geom: The geometry to be opened
    :return: The opened geometry
    """
    return dilate(erode(geom, **kwargs), **kwargs)


@passes_kws_to(shapely.geometry.base.BaseGeometry.buffer)
def pro_closing(geom: BaseGeometry, **kwargs) -> BaseGeometry:
    """
      Remove Salt and Pepper

      Common Variants
    Opening and closing are themselves often used in combination to achieve more subtle results. If we represent the closing of an image f by C(f), and its opening by O(f), then some common combinations include:

    Proper Opening
    Min(f, /em{C}(O(C(f))))

    Proper Closing
    Max(f, O(C(O(f))))

    Automedian Filter
    Max(O(C(O(f))), Min(f, C(O(C(f)))))

    These operators are commonly known as morphological filters.


    Closing
    Dilation means that the central pixel will be replaced by the brightest pixel in the vicinity (filter structural element).
    Perfect for removing pepper noise and ensuring that the key features are relatively sharp.

    Opening
    Erosion means is that if we have a structuring element that is a 3 X 3 matrix, the central pixel will be replaced by the darkest pixel in the 3 X 3 neighborhood.
    Opening is erosion followed by dilation which makes it perfect for removing salt noise (white dots) and ensuring that the key features are relatively sharp.

      :param geom:
      :param kwargs:
      :return:
    """

    return opening(closing(opening(geom, **kwargs), **kwargs), **kwargs)


@passes_kws_to(shapely.geometry.base.BaseGeometry.buffer)
def pro_opening(geom: BaseGeometry, **kwargs) -> BaseGeometry:
    """
      Remove Salt and Pepper

      Common Variants
    Opening and closing are themselves often used in combination to achieve more subtle results. If we represent the closing of an image f by C(f), and its opening by O(f), then some common combinations include:

    Proper Opening
    Min(f, /em{C}(O(C(f))))

    Proper Closing
    Max(f, O(C(O(f))))

    Automedian Filter
    Max(O(C(O(f))), Min(f, C(O(C(f)))))

    These operators are commonly known as morphological filters.


    Closing
    Dilation means that the central pixel will be replaced by the brightest pixel in the vicinity (filter structural element).
    Perfect for removing pepper noise and ensuring that the key features are relatively sharp.

    Opening
    Erosion means is that if we have a structuring element that is a 3 X 3 matrix, the central pixel will be replaced by the darkest pixel in the 3 X 3 neighborhood.
    Opening is erosion followed by dilation which makes it perfect for removing salt noise (white dots) and ensuring that the key features are relatively sharp.

      :param geom:
      :param kwargs:
      :return:
    """

    return closing(opening(closing(geom, **kwargs), **kwargs), **kwargs)


# open = opening # keyword clash
erode = erosion
dilate = dilation
close = closing

if __name__ == "__main__":

    def aishdjauisd():
        # Import constructors for creating geometry collections
        from shapely.geometry import MultiPoint, MultiLineString

        # Import necessary geometric objects from shapely module
        from shapely.geometry import Point, LineString, Polygon

        # Create Point geometric object(s) with coordinates
        point1 = Point(2.2, 4.2)
        point2 = Point(7.2, -25.1)
        point3 = Point(9.26, -2.456)
        # point3D = Point(9.26, -2.456, 0.57)

        # Create a MultiPoint object of our points 1,2 and 3
        multi_point = MultiPoint([point1, point2, point3])

        # It is also possible to pass coordinate tuples inside
        multi_point2 = MultiPoint([(2.2, 4.2), (7.2, -25.1), (9.26, -2.456)])

        # We can also create a MultiLineString with two lines
        line1 = LineString([point1, point2])
        line2 = LineString([point2, point3])
        multi_line = MultiLineString([line1, line2])
        polygon = Polygon([point2, point1, point3])

        from shapely.geometry import GeometryCollection
        from matplotlib import pyplot
        import geopandas

        geoms = GeometryCollection([multi_point, multi_point2, multi_line, polygon])

        # A positive distance produces a dilation, a negative distance an erosion. A very small or zero distance may sometimes be used to “tidy” a polygon.
        geoms = opening(geoms)
        geoms = dilate(geoms)
        geoms = closing(geoms)
        geoms = erode(geoms)

        p = geopandas.GeoSeries(geoms)
        p.plot()
        pyplot.show()

    # aishdjauisd()

    def ahfuashdu():
        from random import random
        import matplotlib.pyplot
        import geopandas

        circle_diameter = 100.0
        ring_width = 6.0

        circle = dilate(shapely.Point(0, 0), distance=circle_diameter)
        ring = circle.difference(erode(circle, distance=ring_width))

        noise_elements = []

        num_noise_points = 1000
        num_noise_lines = 100
        noise_amplitude = 2.0

        for i in range(num_noise_points):
            noise_elements.append(
                dilate(
                    shapely.Point(
                        -circle_diameter + random() * circle_diameter * 2,
                        -circle_diameter + random() * circle_diameter * 2,
                    ),
                    distance=random() * noise_amplitude,
                )
            )

        for i in range(num_noise_lines):
            noise_elements.append(
                dilate(
                    shapely.LineString(
                        [
                            shapely.Point(
                                -circle_diameter + random() * circle_diameter * 2,
                                -circle_diameter + random() * circle_diameter * 2,
                            )
                            for _ in range(2)
                        ]
                    ),
                    distance=random() * noise_amplitude,
                )
            )

        noisy_ring = shapely.unary_union(noise_elements + [ring])

        geopandas.GeoSeries(noisy_ring).plot()
        matplotlib.pyplot.title("noisy_ring")
        matplotlib.pyplot.show()

        some_ring = opening(noisy_ring, distance=noise_amplitude)

        geopandas.GeoSeries(some_ring).plot()
        matplotlib.pyplot.title("opening_ring")
        matplotlib.pyplot.show()

        some_ring = closing(
            opening(noisy_ring, distance=noise_amplitude), distance=noise_amplitude
        )

        geopandas.GeoSeries(some_ring).plot()
        matplotlib.pyplot.title("some_ring")
        matplotlib.pyplot.show()

        pro_closing_ring = pro_closing(noisy_ring, distance=noise_amplitude)

        geopandas.GeoSeries(pro_closing_ring).plot()
        matplotlib.pyplot.title("pro_closing_ring")
        matplotlib.pyplot.show()

        pro_opening_ring = pro_opening(noisy_ring, distance=noise_amplitude)

        geopandas.GeoSeries(pro_opening_ring).plot()
        matplotlib.pyplot.title("pro_opening_ring")
        matplotlib.pyplot.show()

    ahfuashdu()



#!/usr/bin/env python3

from qgis.core import QgsMapLayer, QgsRasterDataProvider
from warg import AlsoDecorator

__all__ = ["QLayerEditSession", "RasterDataProviderEditSession"]


class QLayerEditSession(AlsoDecorator):
    def __init__(self, map_layer: QgsMapLayer):
        self.map_layer = map_layer

    def __enter__(self):
        if self.map_layer:
            self.map_layer.startEditing()
        return True

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.map_layer:
            self.map_layer.commitChanges()


class RasterDataProviderEditSession(AlsoDecorator):
    def __init__(
        self, raster_data_provider: QgsRasterDataProvider, auto_reload: bool = True
    ):
        self.raster_data_provider = raster_data_provider
        self.auto_reload = auto_reload

    def __enter__(self):
        if self.raster_data_provider:
            self.raster_data_provider.setEditable(True)
        return True

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.raster_data_provider:
            self.raster_data_provider.setEditable(False)
            if self.auto_reload:
                self.raster_data_provider.reload()



#!/usr/bin/env python3

__author__ = "Christian Heider Lindbjerg"
__doc__ = r"""

           Created on 02-12-2020
           """

from typing import Tuple

# noinspection PyUnresolvedReferences
from qgis.PyQt import QtWidgets

__all__ = ["make_dialog_progress_bar", "DialogProgressBar"]

from warg import AlsoDecorator, passes_kws_to


class DialogProgressBar(AlsoDecorator):
    @passes_kws_to()
    def __init__(self, progress: int = 0, **kwargs):
        self._progress_dialog, self._progress_bar = make_dialog_progress_bar(
            progress, **kwargs
        )

    def __enter__(self):
        if self._progress_dialog:
            self._progress_dialog.show()
        return self._progress_bar

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self._progress_dialog:
            self._progress_dialog.close()


def make_dialog_progress_bar(
    progress: int = 0,
    *,
    minimum_width: int = 300,
    min_value: int = 0,
    max_value: int = 100,
    title: str = "Progress",
    label: str = ""
) -> Tuple[QtWidgets.QDialog, QtWidgets.QProgressBar]:
    """
    Create a progress bar dialog.

    :param title:
    :param label:
    :param min_value:
    :param max_value:
    :param progress: The progress to display.
    :type progress: int
    :param minimum_width: The minimum width of the dialog.
    :type minimum_width: int
    :return: The dialog.
    :rtype: Tuple[QtWidgets.QDialog, QtWidgets.QProgressBar]
    """
    dialog = QtWidgets.QProgressDialog()
    dialog.setWindowTitle(title)
    dialog.setLabelText(label)

    bar = QtWidgets.QProgressBar(dialog)
    bar.setTextVisible(True)
    bar.setValue(min_value)
    bar.setValue(progress)
    bar.setMaximum(max_value)

    dialog.setBar(bar)
    dialog.setMinimumWidth(minimum_width)

    return dialog, bar


if __name__ == "__main__":

    def calc(x, y):
        from time import sleep

        dialog, bar = make_dialog_progress_bar(0)
        bar.setValue(0)
        bar.setMaximum(100)
        sum_ = 0
        for i in range(x):
            for j in range(y):
                k = i + j
                sum_ += k
            i += 1
            bar.setValue((float(i) / float(x)) * 100)
            sleep(0.1)
        print(sum_)

    # calc(10000, 2000)



#!/usr/bin/env python3

__author__ = "heider"
__doc__ = r"""

           Created on 5/5/22
           """

from pathlib import Path

with open(Path(__file__).parent / "README.md") as this_init_file:
    __doc__ += this_init_file.read()

from .drawing import *
from .environment import *
from .models import *
from .progress_bar import *
from .signals import *
from .timestamp import *
from .sessions import *
from .actions import *



#!/usr/bin/env python3


from enum import Enum

__all__ = [
    "VectorGeometryTypeEnum",
    "CommonCoordinateReferenceSystemEnum",
    "FieldTypeEnum",
]


def QgisWkbToVectorGeometryType():
    ...  # QgsWkbTypes.LineGeometry


class VectorGeometryTypeEnum(Enum):
    point = "point"  # ESRI shapefile
    line_string = "linestring"  # ESRI shapefile
    polygon = "polygon"  # ESRI shapefile
    multi_point = "multipoint"  # ESRI shapefile
    multi_line_string = "multilinestring"  # + Geopackage
    multi_polygon = "multipolygon"  # + Geopackage


class CommonCoordinateReferenceSystemEnum(Enum):
    epsg4326 = "epsg:4326"  # WGS 84
    epsg3857 = "EPSG:3857"  # WGS 84 / Pseudo-Mercator"


class FieldTypeEnum(Enum):
    integer = "integer"
    double = "double"
    string = "string"


def construct_layer_uri() -> str:
    return "Point?crs=epsg:4326&field=id:integer&field=name:string(20)&index=yes"



#!/usr/bin/env python3

from typing import Iterable, Union

from shapely import LineString, LinearRing, MultiLineString, Point

from .projection import line_line_intersection, project_point_to_line

__all__ = [
    "ensure_ccw_ring",
    "ensure_cw_ring",
    "make_projected_ring",
    "make_extruded_ring",
]


def ensure_ccw_ring(ring: LinearRing) -> LinearRing:
    if not ring.is_ccw:
        return LinearRing(list(ring.coords)[::-1])
    return ring


def ensure_cw_ring(ring: LinearRing) -> LinearRing:
    if ring.is_ccw:
        return LinearRing(list(ring.coords)[::-1])
    return ring


def make_projected_ring(
    lines: Union[MultiLineString, Iterable[LineString]], ccw: bool = True
) -> LinearRing:
    points = []

    if isinstance(lines, MultiLineString):
        lines = lines.geoms

    num_lines = len(lines)

    if ccw:
        lines = lines[::-1]

    for n in range(num_lines):
        points.append(project_point_to_line(Point(lines[n - 1].coords[-1]), lines[n]))

    ring = LinearRing(points)

    assert ring.is_closed
    assert ring.is_ring

    return ring


def make_extruded_ring(
    lines: Union[MultiLineString, Iterable[LineString]], ccw: bool = True
) -> LinearRing:
    points = []

    if isinstance(lines, MultiLineString):
        lines = lines.geoms

    num_lines = len(lines)

    if ccw:
        lines = lines[::-1]

    for n in range(num_lines):
        points.append(Point(line_line_intersection(lines[n - 1], lines[n]).coords[-1]))

    ring = LinearRing(points)

    assert ring.is_closed
    assert ring.is_ring

    return ring


if __name__ == "__main__":

    def juijh():
        r"""

            0   1   2

        0   0---0   0
                    |
        1   0       0
            |
        2   0   0---0

        to become


            0   1   2

        0   0---0---0
            |       |
        1   0       0
            |       |
        2   0---0---0

        :return:
        """

        lines = [
            LineString([[0, 0], [1, 0]]),
            LineString([[2, 0], [2, 1]]),
            LineString([[2, 2], [1, 2]]),
            LineString([[0, 2], [0, 1]]),
        ]

        print(make_extruded_ring(lines))

    def juijh2():
        r"""

            0   1   2   3

        0   0---0---0---0
                    |
        1   0       0
            |
        2   0   0---0

        to become


            0   1   2

        0   0---0---0
            |       |
        1   0       0
            |       |
        2   0---0---0

        :return:
        """

        lines = [
            LineString([[0, 0], [3, 0]]),
            LineString([[2, 0], [2, 1]]),
            LineString([[2, 2], [1, 2]]),
            LineString([[0, 2], [0, 1]]),
        ]

        print(make_extruded_ring(lines))

    juijh2()
